from Products.CMFCore.utils import UniqueObject 
from OFS.SimpleItem import SimpleItem
from OFS.PropertyManager import PropertyManager
from Globals import InitializeClass 
from Products.CMFCore.utils import getToolByName
import OFS.Moniker
from OFS.CopySupport import _cb_decode
from AccessControl import ClassSecurityInfo, Unauthorized
from Products.CMFCore import CMFCorePermissions
from config import ADD_ALIAS_PERMISSION


class SimpleAliasTool(UniqueObject, SimpleItem,PropertyManager): 
    """ This tool provides some functions for portlet configuration and management in Plone """ 
    id = 'simplealias_tool' 
    meta_type= 'SimpleAlias tool' 
    plone_tool = 1
        
    manage_options=PropertyManager.manage_options
    security = ClassSecurityInfo()

    def __init__(self):
        self.manage_addProperty('portal_type_filters', '', 'lines')

    security.declarePublic('getLinkableTypes')
    def getLinkableTypes(self, object=None):
        exclude = self.getProperty('portal_type_filters', [])
        portal_types =  getToolByName(self, 'portal_types')
        archetype_tool = getToolByName(self, 'archetype_tool')

        types = [t['portal_type'] for t in archetype_tool.listRegisteredTypes()]
        return tuple([t for t in types if t not in exclude])
        
    
    security.declarePublic('manage_pasteAsAlias')
    def manage_pasteAsAlias(self, context, cb_copy_data=None):

        # first check if the user has the proper permissions in the 'context'
        if not context.portal_membership.checkPermission(ADD_ALIAS_PERMISSION, context):
            raise "Insufficient priviledge to perform this action. It requires the permission: " + ADD_ALIAS_PERMISSION
        
        cp=None
        REQUEST=context.REQUEST

        if cb_copy_data is not None:
            cp = cb_copy_data
        elif REQUEST and REQUEST.has_key('__cp'):
            cp=REQUEST['__cp']
        else:
            cp=None

        if cp is None:
            raise "No cp data"        

        try:
            cp=_cb_decode(cp)
        except:
            raise "can't decode cp: %r" % cp        

        oblist=[]
        app = self.getPhysicalRoot()
        failed=''
        success=''
        types=self.getLinkableTypes()

        for mdata in cp[1]:
            m = OFS.Moniker.loadMoniker(mdata)
            try: ob = m.bind(app)
            except: raise "Objects not found in %s" % app
            oblist.append(ob)        
            
        for ob in oblist:
            if getattr(ob, 'isReferenceable', None) and ob.portal_type in types:
                # create an Alias for this object
                id=context.generateUniqueId('Alias')
                new_id=context.invokeFactory(id=id, type_name='Alias')
                if new_id is None or new_id=='':
                    new_id=id
                o=getattr(context, new_id, None)
                # now set the reference
                o.setAlias(ob.UID())
                o.reindexObject()
                o=None
                success='Alias(es) created. '
            else:
                if failed=='':
                    failed='Ignored (not linkable): ' + ob.Title()
                else:
                    failed=failed + ', ' + ob.Title()

        return success + failed

    def getAliasIcon(self, object):
        objIcon = object.getIcon(relative_to_portal=1)
        aliasIcon = objIcon.replace('.gif', '_alias.gif')

        # check if it exists
        if getattr(self.portal_skins.aq_explicit, aliasIcon, None):
            return aliasIcon
        else:
            return 'alias_icon.gif'    
    
        
InitializeClass(SimpleAliasTool)
